@@ -777,33 +777,3 @@ void R_nc_redef( int *ncid )
 			nc_strerror(err) );
 }
 
-/*********************************************************************/
-void R_nc_enddef( int *ncid )
-{
-	int	err;
-	err = nc_enddef(*ncid);
-	if( err != NC_NOERR ) 
-		fprintf( stderr, "Error in R_nc_enddef: %s\n", 
-			nc_strerror(err) );
-}
-
-/*********************************************************************/
-void R_nc_sync( int *ncid )
-{
-	int	err;
-	err = nc_sync(*ncid);
-	if( err != NC_NOERR ) 
-		fprintf( stderr, "Error in R_nc_sync: %s\n", 
-			nc_strerror(err) );
-}
-
-/*********************************************************************/
-void R_nc_close( int *ncid )
-{
-	int	err;
-	err = nc_close(*ncid);
-	if( err != NC_NOERR ) 
-		fprintf( stderr, "Error in R_nc_close: %s\n", 
-			nc_strerror(err) );
-}
-
